@@ -169,7 +169,7 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
     # They must be compatible to the LockFile interface.
     # A suitable alternative would be the BlockingLockFile
     t_lock = LockFile
-    re_comment = re.compile('^\s*[#;]')
+    re_comment = re.compile(r'^\s*[#;]')
 
     #} END configuration
 
@@ -212,9 +212,9 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         self._is_initialized = False
         self._merge_includes = merge_includes
         self._lock = None
-        self._aquire_lock()
+        self._acquire_lock()
 
-    def _aquire_lock(self):
+    def _acquire_lock(self):
         if not self._read_only:
             if not self._lock:
                 if isinstance(self._file_or_files, (tuple, list)):
@@ -239,7 +239,7 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
         self.release()
 
     def __enter__(self):
-        self._aquire_lock()
+        self._acquire_lock()
         return self
 
     def __exit__(self, exception_type, exception_value, traceback):
@@ -424,7 +424,8 @@ class GitConfigParser(with_metaclass(MetaParserBuilder, cp.RawConfigParser, obje
                     if include_path in seen or not os.access(include_path, os.R_OK):
                         continue
                     seen.add(include_path)
-                    files_to_read.append(include_path)
+                    # insert included file to the top to be considered first
+                    files_to_read.insert(0, include_path)
                     num_read_include_files += 1
                 # each include path in configuration file
             # end handle includes
